<?php

    lt_include( PLOG_CLASS_PATH."class/net/client.class.php" );

define("SUBSCRIBE_POSTS_IN_CATEGORY", 0);
define("SUBSCRIBE_COMMENTS_IN_POST", 1);

class Subscriptions{

        /**
         * Generate confirmation URL and send confirmation email
         */
    function sendConfirmationEmail($blogInfo, $emailAddress){
        $confirmationUrl = Subscriptions::generateConfirmUrl($blogInfo,
                                                     "subscribeConfirm",
                                                     $emailAddress);
        $confirmationCode = Subscriptions::_getConfirmationCode($blogInfo,
                                                                $emailAddress);

        $locale = $blogInfo->getLocale();
        $ipAddress = Client::getIp();
        if(!$ipAddress)
            $ipAddress = "Unknown";
        $ret = Subscriptions::sendMessage($emailAddress,
                                          $locale->tr("subscribe_email_confirm_subject"),
                                          $locale->pr("subscribe_email_confirm_body",
                                                      $confirmationCode,
                                                      $confirmationUrl,
                                                      $ipAddress));
        return $ret;
    }

        /**
         * Generate confirmation URL
         */
    function generateConfirmUrl($blogInfo, $op, $address){
        $confirmationCode = Subscriptions::_getConfirmationCode($blogInfo, $address);

        lt_include(PLOG_CLASS_PATH."class/net/rawrequestgenerator.class.php");
        $rg = new RawRequestGenerator($blogInfo);
        $rg->addParameter("op", $op);
        $rg->addParameter("blogId", $blogInfo->getId());
        $rg->addParameter("code", $confirmationCode);
        $rg->setXHTML(false);
        return ($rg->getIndexUrl().$rg->getRequest());
    }

        /**
         * Reset all confirmation codes in the database
         * Note, this will cause all previously emailed confirmation codes to no longer work,
         * so, this operation should not be called often
         */
    function resetConfirmationCodes($blogInfo){
        lt_include( PLOG_CLASS_PATH."class/database/db.class.php" );
        $prefix = Db::getPrefix();

        $query = "SELECT type,address ".
            "FROM {$prefix}subscribe";

        $db =& Db::getDb();

        $result = $db->Execute($query);
        if(!$result)
            return false;
        
        while($row = $result->FetchRow()){
            $address = $row["address"];
            if(!$code = Subscriptions::_getConfirmationCode($blogInfo, $address))
                return false;
            $query = "UPDATE ${prefix}subscribe SET confirm='$code' WHERE address='$address'";
            if(!$db->Execute($query))
                return false;
        }

        
        return true;
    }

        /**
         * User never typed in a security code, let's make one for him
         */
    function generateNewKey($blogInfo){
        $key = md5(rand().time().filectime(__FILE__));
        $blogSettings = $blogInfo->getSettings();
        $blogSettings->setValue( "plugin_subscribe_key", $key );

        $blogInfo->setSettings( $blogSettings ); 
                        
            // save the blog's settings
        $blogs = new Blogs();
        if(!$blogs->updateBlog($blogInfo))
            return false;
        
        Subscriptions::resetConfirmationCodes($blogInfo);
        return $key;
    }
    
        /**
         * calculate a secret hash used for confirmation of the email address
         */
    function _getConfirmationCode($blogInfo, $address){
        $blogSettings = $blogInfo->getSettings();
        $key = $blogSettings->getValue( "plugin_subscribe_key" );

        if($key == ""){
            if(!($key = Subscriptions::generateNewKey($blogInfo)))
                return false;
        }

        // TODO: once we are on PHP5, use hash() instead        
        return md5($address . $key);
    }
    
        /**
         * Activate a given subscription
         */
    function confirmSubscription($confirmation){
        lt_include( PLOG_CLASS_PATH."class/database/db.class.php" );
        $db =& Db::getDb();
        $prefix = Db::getPrefix();
        
        $query = "UPDATE ${prefix}subscribe SET active=1 WHERE confirm='$confirmation'";
        if(!$db->Execute($query))
            return false;

            // check to make sure that at least one row was updated
        if($db->Affected_Rows() < 1)
            return false;
            
        return true;
    }

        /**
         * add an email address to the database with the
         *  associated type, for each id, but leave it disabled
         * until it is confirmed later
         */
    function addSubscriptions($blogInfo, $ids, $type, $address){
        lt_include( PLOG_CLASS_PATH."class/database/db.class.php" );

        $confirmation = Subscriptions::_getConfirmationCode($blogInfo, $address);

        foreach($ids as $id){
            $db =& Db::getDb();
            $prefix = Db::getPrefix();

            $query = "INSERT IGNORE INTO ${prefix}subscribe (objid, active, type, address, confirm) ".
                "VALUES('$id', 0, '$type', '$address', '$confirmation')";
            $result = $db->Execute($query);
            if(!$result)
                return false;
        }

        return true;
    }

        
        /**
         * remove all subscriptions sent using this confirmation code
         */
    function removeSubscriptions($confirmation){
        lt_include( PLOG_CLASS_PATH."class/database/db.class.php" );

        $db =& Db::getDb();
        $prefix = Db::getPrefix();
        $query = "DELETE FROM ${prefix}subscribe WHERE confirm='$confirmation'";
        $result = $db->Execute($query);
        if(!$result)
            return false;

            // check to make sure that at least one row was updated
        if($db->Affected_Rows() < 1)
            return false;

        return true;
    }

        /**
         * Based on the $type of object, search for $ids
         * and return an array of subscribed email addresses
         */
    function getSubscriptions($ids, $type){
        lt_include( PLOG_CLASS_PATH."class/database/db.class.php" );
        $prefix = Db::getPrefix();

        $query = "SELECT DISTINCT address ".
            "FROM {$prefix}subscribe ".
            "WHERE active=1 AND ".
            "type=".$type." AND (";
        foreach($ids as $id)
            $query .= "objid=$id OR ";
        $query .= "objid=0)";

        $db =& Db::getDb();

        $result = $db->Execute($query);
        if( !$result )
            return Array();
        
        $addresses = Array();
        while($row = $result->FetchRow()){
            $addresses[] = $row["address"];
        }

        return($addresses);
    }
    
    
        /**
         * Sends an email
         * @param to The address to send the email to
         * @param subject The subject of the email
         * @param text The message to be sent
         */
    function sendMessage($to, $subject, $text){
        lt_include(PLOG_CLASS_PATH."class/mail/emailservice.class.php");
        lt_include(PLOG_CLASS_PATH."class/mail/emailmessage.class.php");
        lt_include(PLOG_CLASS_PATH."class/config/config.class.php");
        
            // build the message
        $message = new EmailMessage();
        $config =& Config::getConfig();
        $message->setFrom($config->getValue("post_notification_source_address"));
        $message->setSubject($subject);
        $message->setBody($text);
        $message->addTo($to);
        
            // once the message is ready, send it out
        $service = new EmailService();
        return($service->sendMessage($message));
    }
}

?>